BS.RingSettings = {
  enableRingIntegration: function() {
    this._beforeAjax("enableRingIntegrationForm", "enableRingIntegrationProgress");
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/enable.html', {
      onComplete: function() {
        window.location.reload();
      }
    });
    return false;
  },

  triggerSync: function() {
    this._beforeAjax("ringSyncForm", "ringSyncProgress");
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/triggerSync.html', {
      onComplete: this._refresh
    });
    return false;
  },

  updateHubUrl: function() {
    this._beforeAjax("ringSettingsForm", "hubUrlProgress");
    var that = this;
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/updateHubUrl.html', {
      parameters: {
        hubUrl: $("hubUrlField").value
      },
      onComplete: function(response) {
        var respXML = response.responseXML;
        if (respXML) {
          var handled = BS.XMLResponse.processErrors(respXML, {
            invalidHubUrl: function (elem) {
              $('error_hubUrl').innerText = elem.firstChild.nodeValue;
            }
          });

          if (handled) {
            BS.RingSettings._afterAjaxError("ringSettingsForm", "hubUrlProgress");
            return;
          }
        }

        that._refresh();
      }
    });
    return false;
  },

  registerHubService: function(hubUrl, additionalUrls) {
    this._beforeAjax("ringSettingsForm", "ringActionProgress");
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/registerAndVerifyService.html', {
      parameters: {
        register: 'true',
        hubUrl: hubUrl,
        additionalUrls: additionalUrls
      },
      onComplete: function(response) {
        var respXML = response.responseXML;
        if (respXML) {
          var handled = BS.XMLResponse.processErrors(respXML, {
            accessForbidden: function(elem) {
              $('error_hubUrl').innerText = elem.firstChild.nodeValue;
            },

            registrationFailed: function(elem) {
              $('error_hubUrl').innerText = elem.firstChild.nodeValue;
            },

            verifyUrlError: function(elem) {
              $('error_hubUrl').innerText = elem.firstChild.nodeValue;
            },

            invalidHubUrl: function(elem) {
              $('error_hubUrl').innerText = elem.firstChild.nodeValue;
            }
          });
          if (handled) {
            BS.RingSettings._afterAjaxError("ringSettingsForm", "ringActionProgress");
            return;
          }

          BS.XMLResponse.processRedirect(respXML);
        }
      }
    });
    return false;
  },

  updateHubService: function() {
    this._beforeAjax("ringSettingsForm", "ringUpdateActionProgress");
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/updateService.html', {
      onComplete: this._refresh
    });
    return false;
  },

  unregisterHubService: function() {
    if (confirm("Are you sure you want to unregister the TeamCity service? This will prevent TeamCity users from logging in via Hub.")) {
      this._beforeAjax("ringSettingsForm", "ringActionProgress");
      BS.ajaxRequest(window['base_uri'] + '/hubPlugin/unregisterService.html', {
        onComplete: function() {
          $("ringDiv").refresh();
        }
      });
    }
    return false;
  },

  updateUserFilters: function(showSyncWarningIfUserFilterChanged) {
    var newUserFilter = $("userFilter").value;
    var curUserFilter = $("curUserFilter").value;

    if (showSyncWarningIfUserFilterChanged && curUserFilter != newUserFilter) {
      if (!confirm("You are about to change user filter.\n" +
                  "This may lead to creation or deletion of user accounts in TeamCity during the next synchronization with Hub.\n\n" +
                  "Are you sure?")) return false;
    }

    this._beforeAjax("ringUserFiltersForm", "ringUserFiltersProgress");
    var hasGroupFilter = $("ringUserGroupFilterField") != null;
    BS.ajaxRequest(window['base_uri'] + '/hubPlugin/updateUserFilters.html', {
      parameters: {
        userFilter: newUserFilter,
        userGroupFilter: hasGroupFilter ? $("ringUserGroupFilterField").value : ''
      },
      onComplete: this._refresh
    });
    return false;
  },

  updateSyncSettings: function() {
    var createDeleteUsersEnabled = $("createUsers").checked && ($('curCreateUsers').value != 'true') ||
                                   $('deleteUsers').checked && ($('curDeleteUsers').value != 'true');
    var userSyncEnabled = $('syncUsers').checked && ($('curSyncUsers').value != 'true');

    if (userSyncEnabled && ($("createUsers").checked || $("deleteUsers").checked)) {
      if (!confirm("You are about to enable users synchronization.\n" +
                  "This may lead to creation or deletion of user accounts in TeamCity during the next synchronization with Hub.\n\n" +
                  "Are you sure?")) return false;
    } else if ($('syncUsers').checked && createDeleteUsersEnabled) {
      if (!confirm("You are about to change user synchronization options.\n" +
                  "This may lead to creation or deletion of user accounts in TeamCity during the next synchronization with Hub.\n\n" +
                  "Are you sure?")) return false;
    }

    var form = OO.extend(BS.AbstractWebForm, {
      formElement: function() {
        return $('ringSyncSettingsForm');
      }
    });

    this._beforeAjax("ringSyncSettingsForm", "ringSyncSettingsProgress");
    BS.FormSaver.save(form, window['base_uri'] + '/hubPlugin/updateSyncSettings.html', OO.extend(BS.ErrorsAwareListener, {
      onSuccessfulSave: this._refresh
    }));

    return false;
  },

  _beforeAjax: function(formName, progressId) {
    $j("#ringDiv").trigger("beforeRefresh");
    BS.Util.disableFormTemp($(formName));
    BS.Util.show(progressId);
  },

  _afterAjaxError: function(formName, progressId) {
    BS.Util.reenableForm($(formName));
    BS.Util.hide(progressId);
  },

  _refresh: function() {
    $("ringDiv").refresh(null, null, function () {
      $j("#ringDiv").trigger("afterRefresh");
    });
  }
};

$j(document).on("updateUserFilters", "#ringDiv", BS.RingSettings.updateUserFilters.bind(BS.RingSettings));